#ifndef _HGLSPLATRENDERER_H_
#define _HGLSPLATRENDERER_H_

#include "../../../DataTypes/src/MyDataTypes.h"
#include "../../../Core/Interfaces/src/RendererInterface.h"
#include "../../../Core/Interfaces/src/RendererConfigurationInterface.h"
#include "../../../Core/Interfaces/src/FrameBufferInterface.h"
#include "HGLSplatRendererWidget.h"
#include "HGLSplatRendererConfiguration.h"

/**
 * This class implements the <code>RendererInterface</code> and acts as a wrapper class
 * for the HGLSplat renderer in the <code>HGLSplatRendererWidget</code>
 *
 * @author Matthias Zwicker
 * @version 1.2
 * @see HGLSplatRendererWidget
 */
class HGLSplatRenderer : public RendererInterface {

	Q_OBJECT

public:

	HGLSplatRenderer();
	virtual ~HGLSplatRenderer();

	// ***************************
	// interface RendererInterface
	// ***************************

	RENDERER_API void initialize (const bool isVisible, QWidget *parent);
	RENDERER_API void setVisible (const bool enable);
	RENDERER_API bool isVisible() const;
	RENDERER_API bool isInteractive() const;
	RENDERER_API bool providesPointPicking() const;
	RENDERER_API float getDepth (unsigned int x, unsigned int y) const ;
	RENDERER_API Vector3D getCameraSpaceNormal (unsigned int x, unsigned int y) const;

	RENDERER_API QWidget *getRendererWidget() const;
	RENDERER_API QImage getRenderedImage();
	RENDERER_API RendererConfigurationInterface *getRendererConfiguration();

	RENDERER_API void setSceneView (const MyDataTypes::TransformationMatrix16f newSceneViewMatrix);
	RENDERER_API void getSceneView (MyDataTypes::TransformationMatrix16f sceneViewMatrix) const;
	RENDERER_API void setViewFrustum (const MyDataTypes::ViewFrustum newViewFrustum);
	RENDERER_API MyDataTypes::ViewFrustum getViewFrustum() const;
	RENDERER_API void setViewPortSize (const QSize newViewPortSize);
	RENDERER_API QSize getViewPortSize() const;
	RENDERER_API void setShadingEnabled (const bool enable);
	RENDERER_API bool isShadingEnabled() const;
	RENDERER_API void setLightDirection (const Vector3D newLightDirection);
	RENDERER_API Vector3D getLightDirection() const;
	RENDERER_API void setBackgroundColor(QRgb newBackgroundColor);
	RENDERER_API QRgb getBackgroundColor() const;
	RENDERER_API void setTwoSidedNormalsEnabled(const bool enable);
	RENDERER_API bool isTwoSidedNormalsEnabled() const;
	RENDERER_API void setShadowsEnabled (const bool enable);
	RENDERER_API bool isShadowsEnabled() const;
	RENDERER_API void toggleSelectionVisualization();
	RENDERER_API void setSelectionVisualizationEnabled (const bool enable);
	RENDERER_API bool isSelectionVisualizationEnabled() const;
	RENDERER_API void setCutoffRadius (const float radius);
	RENDERER_API float getCutoffRadius() const;
	RENDERER_API void setBlendingThresholds (const float constThreshold, const float distThreshold, const float angleThreshold);
	RENDERER_API void getBlendingThresholds (float *constThreshold, float *distThreshold, float *angleThreshold) ;

	
	RENDERER_API void renderFrame (const int = (FrameBufferInterface::PERPIXEL_C_Z_N_W | FrameBufferInterface::PERPIXEL_OBJECTPOINTER), const bool allScene = true);	
	RENDERER_API void addToFrame (const std::vector<SurfelInterface *>* surfels, int nSurfels, const int attributes = FrameBufferInterface::PERPIXEL_C_Z_N_W, const bool updateRendererWidget = true);
	RENDERER_API void subtractFromFrame (const std::vector<SurfelInterface *>* surfels, int nSurfels, const int attributes = FrameBufferInterface::PERPIXEL_C_Z_N_W , const bool updateRendererWidget = true);

	// *******************************************
	// not supported (not an interactive renderer)
	// *******************************************

	RENDERER_API FrameBufferInterface *getFrameBuffer() const;

		
private:

	HGLSplatRendererWidget        *rendererWidget;
	HGLSplatRendererConfiguration rendererConfiguration;
  
private slots:

	// connected to the HGLSplatRendererWidget signal repainted: emits the signal rendererWidgetRepainted
	void handleRepainted();
};

// ****************
// access functions
// ****************

// extern "C" is needed so those functions can be accessed
// with GetProcAddress() (WIN32) and dlsym() (UNIX) respective
extern "C" {
	RENDERER_API RendererInterface *createRenderer();
	RENDERER_API const char *getRendererType();
}

#endif  // _HGLSPLATRENDERER_H_
