#ifndef _HGLSPLATRENDERERCONFIGURATION_H_
#define _HGLSPLATRENDERERCONFIGURATION_H_

#include <qwidget.h>

#include "../../../Core/Interfaces/src/RendererConfigurationInterface.h"
#include "HGLSplatRendererConfigurationWidget_Impl.h"

/**
 * This class implements the settings for the HGLSplat renderer.
 *
 * @author Matthias Zwicker
 * @version 1.2
 */
class HGLSplatRendererConfiguration : public RendererConfigurationInterface {

	Q_OBJECT

public:

	HGLSplatRendererConfiguration();
	virtual ~HGLSplatRendererConfiguration();

	void setCutOffRadius (const float newCutoffRadius);
	float getCutOffRadius() const;
	void setConditionThreshold (const float newConditionThreshold);
	float getConditionThreshold() const;
	void setDepthOffset(const float newDepthOffset);
	float getDepthOffset() const;
	QString getCgProgramDirectory() const;

	// ****************************************
	// interface RendererConfigurationInterface
	// ****************************************

	QWidget *createConfigurationWidget (QWidget *parent);
	void apply();

signals:

	void cutOffRadiusChanged (const float newCutOffRadius);
	void conditionThresholdChanged(const float newConditionThreshold);
	void depthOffsetChanged(const float newDepthOffset);
	void reloadCGPrograms ();
	void setBlendingEnabled(bool enable);

private slots:
	void handleReloadCGPrograms();

private:

	float cutOffRadius;
	float conditionThreshold;
	float depthOffset;
	QString programDirectory;
	HGLSplatRendererConfigurationWidget_Impl *configurationWidget;

};

#endif  // _HGLSPLATRENDERERCONFIGURATION_H_
